SystemOrganization addCategory: #'StashShell-NIX'!

StashCommand subclass: #StashNIXBuiltInCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-NIX'!

!StashNIXBuiltInCommand classSide methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 16:00'!
commandString

	self subclassResponsibility! !

!StashNIXBuiltInCommand classSide methodsFor: 'instance creation' stamp: 'DaleHenrichs 12/9/2010 15:59'!
new

	^(super new)
		command: self commandString; 
		yourself! !

StashNIXBuiltInCommand subclass: #StashNIXcdCommand
	instanceVariableNames: 'directories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-NIX'!

!StashNIXcdCommand classSide methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 18:04'!
commandString

	^'cd'! !

!StashNIXcdCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:02'!
directories

	^directories! !

!StashNIXcdCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:50'!
executeAgainst: aStashShellDevice in: aStashShell
	self helpFlag ifTrue: [ ^self usageMessage ].
	aStashShell
		path:
			([ aStashShell resolvePathFromPathString: self directories first ]
				on: StashShellNoSuchChild
				do: [ :ex | ex resume: false ]).
	^ ''! !

!StashNIXcdCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:24'!
processArgs: argsString
	| args |
	args := self parse: argsString.
	args isPetitFailure
		ifTrue: [ ^ self argsParsingError ].
	args isEmpty
		ifTrue: [ ^ self missingOperandError ].
	directories := OrderedCollection new.
	self getopts: args into: directories.
! !

!StashNIXcdCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:12'!
selector

	^self subclassResponsibility! !

!StashNIXcdCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:53'!
usage

	^'cd [-dfh] [dir]'! !

!StashNIXcdCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:53'!
usageMessage
	"cd [-dfh] [dir]"

	^self usage, '
	Change current working directory to the folder specified in the <dir> path.

	-d bring up debugger on error instead of printing error message
	-f cd into the file device (inspect).
	-h usage message.
'! !

StashNIXBuiltInCommand subclass: #StashNIXhelpCommand
	instanceVariableNames: 'commands'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-NIX'!

!StashNIXhelpCommand classSide methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 16:00'!
commandString

	^'help'! !

!StashNIXhelpCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:19'!
commands

	^commands! !

!StashNIXhelpCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:50'!
executeAgainst: aStashShellDevice in: aStashShell
	self helpFlag ifTrue: [ ^self usageMessage ].
	self commands isEmpty
		ifTrue: [ ^ self helpText: aStashShell ].
	self halt! !

!StashNIXhelpCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/11/2010 12:34'!
helpText: aShell
	| stream |
	stream := WriteStream on: String new.
	stream
		nextPutAll: 'Smaltalk shell, version ' , ConfigurationOfStashShell project currentVersion versionString;
		cr;
		nextPutAll: 'These shell commands are defined internally.';
		cr;
		nextPutAll: 'Type ''help'' to see this list.';
		cr;
		nextPutAll: 'Type ''help name'' to find out more about the function ''name''.';
		cr;
		nextPutAll: 'Use ''info stash'' to find out more about she shell in general.';
		cr.
	(aShell builtinCommands values asArray sort: [ :a :b | a name <= b name ])
		do: [ :cmd | 
			stream
				cr;
				nextPutAll: cmd usage ].
	^ stream contents! !

!StashNIXhelpCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:24'!
processArgs: argsString
	| args |
	args := self parse: argsString.
	args isPetitFailure
		ifTrue: [ ^ self argsParsingError ].
	commands := OrderedCollection new.
	self getopts: args into: commands.
! !

!StashNIXhelpCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:53'!
usage

	^'help [-dh] [functionName]'! !

StashNIXBuiltInCommand subclass: #StashNIXlsCommand
	instanceVariableNames: 'files'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-NIX'!

!StashNIXlsCommand classSide methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 16:00'!
commandString

	^'ls'! !

!StashNIXlsCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:50'!
executeAgainst: aStashShellDevice in: aStashShell
	| stream first showNodeName pathSize |
	self helpFlag ifTrue: [ ^self usageMessage ].
	self files isEmpty
		ifTrue: [ ^ aStashShellDevice listContents ].
	stream := WriteStream on: String new.
	first := true.
	showNodeName := files size > 1.
	pathSize := files size.
	1 to: pathSize do: [ :pathIndex | 
		| parentFolder |
		[ 
		parentFolder := (aStashShell resolvePathFromPathString: (files at: pathIndex)) last.
		first
			ifFalse: [ stream cr ].
		first := false.
		showNodeName
			ifTrue: [ 
				stream
					nextPutAll: parentFolder name , ':';
					cr ].
		parentFolder isFolder
			ifTrue: [ stream nextPutAll: parentFolder listContents ]
			ifFalse: [ stream nextPutAll: parentFolder name ].
		pathSize > pathIndex
			ifTrue: [ stream cr ] ]
			on: Error
			do: [ :ex | 
				self debugFlag
					ifTrue: [ self halt ].
				stream nextPutAll: self command , ': ' , self argumentsString , ': ' , ex messageText ] ].
	^ stream contents! !

!StashNIXlsCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:06'!
files

	^files! !

!StashNIXlsCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:25'!
processArgs: argsString
	| args |
	args := self parse: argsString.
	args isPetitFailure
		ifTrue: [ ^ self argsParsingError ].
	files := OrderedCollection new.
	self getopts: args into: files.
! !

!StashNIXlsCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:51'!
usage

	^'ls [-adh] [file ...]'! !

!StashNIXlsCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:52'!
usageMessage
	"ls [-adh] [file ...]"

	^'Usage: ', self usage, '
	List contents of folder.

	-a list instance variables of folder and contents
	-d bring up debugger on error instead of printing error message
	-h usage message.
'! !

StashNIXBuiltInCommand subclass: #StashNIXmkdirCommand
	instanceVariableNames: 'directories'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-NIX'!

!StashNIXmkdirCommand classSide methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 16:00'!
commandString

	^'mkdir'! !

!StashNIXmkdirCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:07'!
directories

	^directories! !

!StashNIXmkdirCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:50'!
executeAgainst: aStashShellDevice in: aStashShell
	| childNode argCount argIndex stream first |
	self helpFlag ifTrue: [ ^self usageMessage ].
	stream := WriteStream on: String new.
	first := true.
	self directories
		do: [ :pathString | 
			[ 
			| path parentFolder folder |
			path := [ aStashShell resolvePathFromPathString: pathString ]
				on: StashShellNoSuchChild
				do: [ :ex | ex resume: false ].
			folder := path removeLast.
			parentFolder := path last.
			parentFolder mkdir: folder name ]
				on: Error
				do: [ :ex | 
					self debugFlag
						ifTrue: [ self halt ].
					first
						ifFalse: [ stream cr ].
					first := false.
					stream nextPutAll: self command , ': ' , self argumentsString , ': ' , ex messageText ] ].
	^ stream contents! !

!StashNIXmkdirCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:25'!
processArgs: argsString
	| args |
	args := self parse: argsString.
	args isPetitFailure
		ifTrue: [ ^ self argsParsingError ].
	args isEmpty
		ifTrue: [ ^ self missingOperandError ].
	directories := OrderedCollection new.
	self getopts: args into: directories.
! !

!StashNIXmkdirCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:53'!
usage

	^'mkdir [-dh] dir...'! !

StashNIXBuiltInCommand subclass: #StashNIXmountCommand
	instanceVariableNames: 'deviceList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-NIX'!

!StashNIXmountCommand classSide methodsFor: 'accessing' stamp: 'DaleHenrichs 12/11/2010 13:39'!
commandString

	^'mount'! !

!StashNIXmountCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:09'!
deviceList

	^deviceList! !

!StashNIXmountCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:50'!
executeAgainst: aStashShellDevice in: aStashShell
	| devPath dirPath device mountPoint directory |
	self helpFlag ifTrue: [ ^self usageMessage ].
	devPath := aStashShell resolvePathFromPathString: (self deviceList at: 1).
	dirPath := [ aStashShell resolvePathFromPathString: (self deviceList at: 2) ]
		on: StashShellNoSuchChild
		do: [ :ex | ex resume: false ].
	device := devPath last object.
	mountPoint := dirPath removeLast.
	directory := dirPath last object.
	directory isNIXDirectory
		ifFalse: [ self halt ].
	device isMountable
		ifFalse: [ self halt ].
	device name: mountPoint name.
	directory addChild: device.
	^ ''! !

!StashNIXmountCommand methodsFor: 'execution' stamp: 'DaleHenrichs 12/12/2010 09:25'!
processArgs: argsString
	| args |
	args := self parse: argsString.
	args isPetitFailure
		ifTrue: [ ^ self argsParsingError ].
	deviceList := OrderedCollection new.
	self getopts: args into: deviceList.
! !

!StashNIXmountCommand methodsFor: 'accessing' stamp: 'DaleHenrichs 12/12/2010 09:53'!
usage

	^'mount [-dh] device mount_point'! !

StashVirtualMachine subclass: #StashNIXMachine
	instanceVariableNames: 'builtinCommands'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-NIX'!

!StashNIXMachine methodsFor: 'accessing' stamp: 'DaleHenrichs 12/11/2010 09:57'!
builtinCommands
	builtinCommands
		ifNil: [ 
			builtinCommands := Dictionary new.
			StashNIXBuiltInCommand subclasses do: [ :cmdClass | builtinCommands at: cmdClass commandString asSymbol put: cmdClass new ] ].
	^ builtinCommands! !

!StashNIXMachine methodsFor: 'accessing' stamp: 'DaleHenrichs 12/11/2010 14:21'!
defaultRoot
	| directory |
	(directory := (StashNIXDirectory new)
		name: '/';
		yourself)
		addChild: StashNIXdev new;
		yourself.
	^ directory! !

!StashDevice methodsFor: '*stashshell-nix-builtins' stamp: 'DaleHenrichs 12/11/2010 15:05'!
isNIXDirectory

	^false! !

!StashDevice methodsFor: '*stashshell-nix-builtins' stamp: 'DaleHenrichs 12/11/2010 12:37'!
listContents
	^self listContentsWithOptions: #()! !

!StashDevice methodsFor: '*stashshell-nix-builtins' stamp: 'DaleHenrichs 12/11/2010 12:21'!
listContentsWithOptions: options
	self subclassResponsibility! !

!StashDevice methodsFor: '*stashshell-nix-builtins' stamp: 'DaleHenrichs 12/11/2010 12:21'!
mkdir: childName

	self subclassResponsibility! !

StashDevice subclass: #StashNIXDirectory
	instanceVariableNames: 'children'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-NIX'!

!StashNIXDirectory methodsFor: 'accessing' stamp: 'DaleHenrichs 12/11/2010 13:19'!
addChild: aStashChildNode

	aStashChildNode parent: self.
	self children at: aStashChildNode name put: aStashChildNode.
	^aStashChildNode! !

!StashNIXDirectory methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 14:32'!
children

	children ifNil: [ children := Dictionary new ].
	^ children! !

!StashNIXDirectory methodsFor: 'accessing' stamp: 'DaleHenrichs 12/9/2010 14:31'!
children: anObject
	children := anObject! !

!StashNIXDirectory methodsFor: 'testing' stamp: 'DaleHenrichs 12/11/2010 15:07'!
isNIXDirectory

	^true! !

!StashNIXDirectory methodsFor: 'builtins' stamp: 'DaleHenrichs 12/11/2010 16:04'!
listContentsWithOptions: options
	| stream first |
	stream := WriteStream on: String new.
	first := true.
	(options includes: #full)
		ifTrue: [ 
			stream nextPutAll: '.'.
			first := false.
			self parent
				ifNotNil: [ 
					stream
						cr;
						nextPutAll: '..' ] ].
	(self children values asArray sort: [ :a :b | a name <= b name ])
		do: [ :child | 
			first
				ifFalse: [ stream cr ].
			stream nextPutAll: child name.
			first := false ].
	^ stream contents! !

!StashNIXDirectory methodsFor: 'builtins' stamp: 'DaleHenrichs 12/11/2010 13:19'!
mkdir: childName
	^(self hasChildNamed: childName)
		ifTrue: [ self error: 'cannot create directory: ', childName printString , ': File exists' ]
		ifFalse: [ 
			self
				addChild:
					((self class new)
						name: childName;
						yourself) ]! !

StashDevice subclass: #StashNIXdev
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'StashShell-NIX'!

!StashNIXdev methodsFor: 'accessing' stamp: 'DaleHenrichs 12/11/2010 15:28'!
children
	| dict |
	dict := Dictionary new.
	(StashDevice subclasses select: [ :cl | cl isMountable ]) collect: [ :cl | dict at: cl mountName put: cl new ].
	^ dict! !

!StashNIXdev methodsFor: 'builtins' stamp: 'DaleHenrichs 12/11/2010 15:28'!
listContentsWithOptions: options
	| stream first |
	stream := WriteStream on: String new.
	first := true.
	(self children values asArray
		sort: [ :a :b | a name < b name ])
		do: [ :dev | 
			first
				ifFalse: [ stream cr ].
			first := false.
			stream nextPutAll: dev name ].
	^ stream contents! !

!StashNIXdev methodsFor: 'accessing' stamp: 'DaleHenrichs 12/11/2010 14:20'!
name
	^'dev'! !

!StashNIXdev methodsFor: 'accessing' stamp: 'DaleHenrichs 12/11/2010 15:46'!
shellDeviceClass

	^StashShellFile! !

!StashShellDevice methodsFor: '*stashshell-nix-builtins' stamp: 'DaleHenrichs 12/11/2010 12:13'!
listContentsWithOptions: options
	^self object listContentsWithOptions: options! !

!StashShellFile methodsFor: '*stashshell-nix-builtins' stamp: 'DaleHenrichs 12/11/2010 15:36'!
listContentsWithOptions: options
	^self shouldNotImplement! !

!StashShellFolder methodsFor: '*stashshell-nix-builtins' stamp: 'DaleHenrichs 12/11/2010 15:37'!
mkdir: childName

	^self object mkdir: childName
! !
